from __future__ import print_function
import os
import sys
import datetime
from xml.etree.ElementTree import ElementTree,fromstring,XML,tostring

"""
Installer of UTSUSEMI
To install UTSUSEMI environment into Ubuntu 12.04, 14.04 and 16.04 LTS with Manyo release version
VerC0.3.8    : Utsusemi4 
VerC0.3.7    : Python 3 only
VerC0.3.6    : DNA Enabled
               Note : Ubuntu packages 'libblas-dev' and 'liblapack-dev' required

VerC0.3.3.3  : bugfix for Default Installation
verC0.3.3.2  : bugfix for SAS (171227)
verC0.3.3.1  : for pyqtgraph
verC0.3.3.0-2: for Ubuntu 16.04LTS
verC0.3.3.0-1: bugfix the issue when overwrite update
verC0.3.3.0 : for release 0.3.3.0
"""
__version__ = "C0.3.8"

############################################################
class InstallToUbuntu:
    ########################################################
    def __init__(self,debug=False):
        """
        Constructor
        @param debug (bool) if debug=True, installer does not touch disk
        @return None
        """
        self.debug = debug
        self.logs = []
        self.logs_index = 0
        self.DefInstBL = 1
        self.cdir = os.getcwd()
        self.MLFDEVROOT = os.path.join( os.environ["HOME"],".MLFENV" )
        if not os.path.exists( self.MLFDEVROOT ):
            os.system( "mkdir %s" % self.MLFDEVROOT )
        self.MLFDEVENVFILE = os.path.join( self.MLFDEVROOT,"environ.xml" )
        
        self.PathDic = {}
        self.PathDic["mlfsoft_root"] = ""  ## for mlfsoft
        self.PathDic["data_root"] = ""     ## for /data/SIK or /data/AMR
        self.PathDic["userhome"] = ""      ## for ~/ana/ and ~/exp/
        self.PathDic["mlfenv"] = ""        ## 
        self.PathDic["environfile"] = ""   
        
        self.MakeDic = {}
        self.MakeDic["CXX"] = ""
        self.MakeDic["PYVER"] = ""
        self.MakeDic["NUM_OF_THREADS"] = ""
        
        self.InstallList = []
        
        self._initialEnviron()
        
        self.dom = None
        
        self.date_fomat = "[%4d/%2d/%2d %2d:%2d:%2d] "
        #self._checkfiles()

    ########################################################
    def _initialEnviron(self):
        self.PathDic["mlfsoft_root"] = "/opt/mlfsoft"  ## for mlfsoft
        self.PathDic["data_root"] = "/data"            ## for /data/SIK or /data/AMR
        self.PathDic["userhome"] = os.environ["HOME"]  ## for ~/ana/ and ~/exp/
        
        self.MakeDic["CXX"] = "g++"
        sysver=sys.version.split(" ")[0]
        sysver_list=sysver.split(".")
        self.MakeDic["PYVER"] = "%1s.%1s"%(sysver_list[0],sysver_list[1])
        self.MakeDic["NUM_OF_THREADS"] = 8

    ########################################################
    def _readEnviron(self):
        """
        """
        if not os.path.exists( self.MLFDEVENVFILE ):
            msg = " There is no saved environment file (%s)." % self.MLFDEVENVFILE
            print(msg)
            self._writeLog(msg)
            self._saveEnviron()
        else:
            fd = open( self.MLFDEVENVFILE,'r')
            dom=ElementTree(file=fd)
            fd.close()

            root_ele = dom.getroot()

            try:
                self.PathDic["mlfsoft_root"] = root_ele.find("mlfSoftRoot").text
                self.PathDic["data_root"] = root_ele.find("mlfBaseDataPath").text
                self.PathDic["userhome"] = root_ele.find("mlfUserHome").text
                self.DefInstBL = int( root_ele.find("mlfInstBL").text )
            except:
                msg = "Invalid parameter file."
                print(msg)
                self._writeLog(msg)

        #self._showEnviron()
        
    ########################################################
    def _showEnviron(self):
        
        print("----------------------------------------------------------")
        print(" Install Directories")
        print("    MLF software root (Manyo, Utsusemi): %s" % self.PathDic["mlfsoft_root"])
        print("    User's home directory              : %s" % self.PathDic["userhome"])
        print("    RAW data  Directory                : %s" % self.PathDic["data_root"])
        print("----------------------------------------------------------")


    ########################################################
    def _saveEnviron(self):
        """
        """
        def s_tab( x ):
            kk = ""
            for i in range(x):
                kk += "    "
            return kk
        
        ss = "<utsusemiOnUbuntuEnviron>\n"
        ss += s_tab(1)+("<mlfSoftRoot>%s</mlfSoftRoot>\n" % (self.PathDic["mlfsoft_root"]))
        ss += s_tab(1)+("<mlfBaseDataPath>%s</mlfBaseDataPath>\n" % (self.PathDic["data_root"]))
        ss += s_tab(1)+("<mlfUserHome>%s</mlfUserHome>\n" % (self.PathDic["userhome"]))
        ss += s_tab(1)+("<mlfInstBL>%d</mlfInstBL>\n" % (self.DefInstBL))
        ss += "</utsusemiOnUbuntuEnviron>\n"

        xml = XML( ss )
        try:
            fd = open( self.MLFDEVENVFILE,"w" )
            fd.write( '<?xml version="1.0" encoding="utf-8"?>\n' )
            tmp=str( tostring(xml,encoding="unicode") )
            fd.write( tmp )
            fd.close()
        except:
            msg = "Cannot open %s for saving instalation parameters." % self.MLFDEVENVFILE
            print(msg)
            self._writeLog(msg)

    ########################################################
    def _checkfiles(self, foldername):
        """
        Check directory or file name required for install
        @param  foldername
        @return None
        """
        self.cdir = os.getcwd()
        src_dir = os.path.join( self.cdir, "src" )
        dlist = os.listdir( src_dir )
        
        isExist = False
        for item in dlist:
            if (item.find( str( foldername ) )!=-1 ) and (os.path.isdir( os.path.join( src_dir, item ) ) ):
                isExist = True
                break
            else:
                msg = "There is no source directory for %s."%( foldername )
                isExist = False
        
        return isExist
    
    ########################################################
    def uninstall(self,delDataFlag=False):
        """
        Uninstall procedure
        @param delDataFlag (bool) if True, delete Raw data and xml files
        @retval 0  No problem
        @retval -1 Error happened
        """

        self._readEnviron()
        
        #InstCode = self.DicInstCode[ self.DefInstBL ]
        #InstDir = os.path.join(self.IROHAroot,('mlf_%s' % InstCode ) )
        coms = []
        coms.append("sudo rm -rf %s" % ( self.PathDic["mlfsoft_root"] ) )
        msg = 'self.PathDic["mlfsoft_root"]= %s' % self.PathDic["mlfsoft_root"]
        self._writeLog(msg)
        
        #coms.append("sudo rm -f /etc/ld.so.conf.d/manyo.conf")
        
        if delDataFlag:
            coms.append("rm -rf %s" % os.path.join( self.PathDic["userhome"],"ana" ) )
            coms.append("rm -rf %s" % os.path.join( self.PathDic["userhome"],"exp" ) )
            coms.append("sudo rm -rf %s" % self.PathDic["data_root"] )

        flist = os.listdir( self.cdir )
        for item in flist:
            if (item.find("manyo")!=-1) and (os.path.isdir(item)):
                coms.append("rm -rf manyo*")
            if (item.find("python")!=-1) and (os.path.isdir(item)):
                coms.append("rm -rf python-*")
            if (item.find("do_make_Utsusemi.py")!=-1):
                coms.append("rm -rf do_make*.py")
            
        try:
            for com in coms:
                os.system(com)
                self._writeLog(com)
        except:
            return -1

        return 0
        

    ########################################################
    def AskPrivateInstallPath(self):
        """

        @param None
        @retval  0    OK
        @retval -1    do nothing
        """

        self.PathDic["mlfsoft_root"] = os.path.join( os.environ["HOME"], "mlfsoft" )
        self.PathDic["data_root"] = os.path.join( os.environ["HOME"], "mlfsoft", "RAWDATA" )
        self.PathDic["userhome"] = os.environ["HOME"]
        
        print("============================================================")
        print("                   Set your install paths")
        while( True ):
            print(" ")
            print("============================================================")
            print(" 1. Path to install Utsusemi software")
            print("   --> %s" % self.PathDic["mlfsoft_root"])
            print(" ")
            print(" 2. Path to Event data directory")
            print("   --> %s" % self.PathDic["data_root"])
            print(" ")
            print(" 3. Path to user directory (for ana/xml or exp/xml)")
            print("   --> %s" % self.PathDic["userhome"])
            print("============================================================")
            print(" ")
            ret=input(">> Which do you change ? [1 to 3. 0 to exit] ")
            if ret=="1":
                r_dir = self._AskNewPath("install Utsusemi software")
                if r_dir!="":
                    self.PathDic["mlfsoft_root"] = r_dir
                continue
            elif ret=="2":
                r_dir = self._AskNewPath("Event data directory")
                if r_dir!="":
                    self.PathDic["data_root"] = r_dir
                continue
            elif ret=="3":
                r_dir = self._AskNewPath("User directory")
                if r_dir!="":
                    self.PathDic["userhome"] = r_dir
                continue
            elif ret=="0":
                break
            else:
                print("Input value is invalid.")
        
        return 0
    ########################################################
    def _AskNewPath(self,title):
        """
        """
        print("Please imput new path to %s = " % title)
        path_in=input(">> ")
        if path_in=="":
            return ""
        
        coms = []
        coms.append( "mkdir -p %s"%(path_in) )
        if -1 == self._doCommands( coms, self.debug ):
            return ""
        
        return path_in
        """
        ret = self._makeNewDir( path_in )
        if ret==-1:
            return ""
        
        return ret
        """

    ########################################################
    def installUbuntuPackages(self):
        """
        Install Ubuntu Package to be required for installation
        @retval 0  No problem
        @retval -1 Error happend
        """
        import subprocess
        tmp = str( subprocess.check_output(["cat","/etc/issue"], universal_newlines=True) )
        tmp_l = tmp.split(" ")
        major_ver_ubuntu = tmp_l[1][:2]

        pkg_list = ['g++','swig',
                    'python3-dev',
                    'libhdf5-dev',
                    'libmxml-dev',
                    'libgsl-dev',
                    'python3-numpy','python3-scipy','python3-matplotlib'
                    ]

        after_pkg_list = []
        if major_ver_ubuntu=="16":
            pkg_list.append( 'libnexus0-dev' )
            pkg_list.append( 'libblas-dev' )
            pkg_list.append( 'liblapack-dev' )
            pkg_list.append( "libboost-all-dev" )
            pkg_list.append( 'python3-pip' )
            #pkg_list.remove( 'python-wxgtk2.8' )
            #pkg_list.append( 'python-wxgtk3.0' )
            #pkg_list.append( 'python-pip' )
            #pkg_list.append( 'python-qt4-gl' )
            #pkg_list.append( 'python-qt4' )
            #pkg_list.append( 'python-opengl' )
            after_pkg_list.append( 'python3-pyside' )
            after_pkg_list.append( 'python3-pyqt4' )
            after_pkg_list.append( 'python3-pyqt4.qtopengl' )
            after_pkg_list.append( 'python3-pyqtgraph' )
        elif major_ver_ubuntu=="18":
            pkg_list.append( 'make' )
            pkg_list.append( 'libnexus0-dev' )
            pkg_list.append( 'libblas-dev' )
            pkg_list.append( 'liblapack-dev' )
            pkg_list.append( 'libboost-dev' )
            pkg_list.append( 'libboost-serialization-dev' )
            pkg_list.append( 'libboost-filesystem-dev' )
            pkg_list.append( 'libboost-system-dev' )
            pkg_list.append( 'libboost-program-options-dev' )
            pkg_list.append( 'python3-pip' )
            pkg_list.append( 'python3-pyside' )
            pkg_list.append( 'python3-pyqt4' )
            pkg_list.append( 'python3-pyqt4.qtopengl' )
            pkg_list.append( 'python3-pyqtgraph' )
            pkg_list.append( 'libcanberra-gtk-module' )

        elif major_ver_ubuntu=="20":
            pkg_list.append( 'libnexus-dev' )
            pkg_list.append( 'libboost-dev' )
            pkg_list.append( 'libboost-serialization-dev' )
            pkg_list.append( 'libboost-filesystem-dev' )
            pkg_list.append( 'libboost-system-dev' )
            pkg_list.append( 'libboost-program-options-dev' )
            pkg_list.append( 'python3-pyside2.qtcore' )
            pkg_list.append( 'python3-pyside2.qtwidgets' ) # pkg_list.append( 'python3-pyside2.qtgui' )
            
        else:
            msg="Ubuntsu Linux version is not suitable."
            raise UserWarning(msg)
        
        coms = []
        if major_ver_ubuntu=="16":
            for pkg in pkg_list:
                coms.append( ( 'sudo apt-get install -y %s' % pkg) )
            coms.append( 'sudo pip3 install --upgrade pip' )
            coms.append( 'sudo pip3 install --upgrade PyOpenGL' )
        elif major_ver_ubuntu=="18":
            for pkg in pkg_list:
                coms.append( ( 'sudo apt install -y %s' % pkg) )
            coms.append( 'sudo pip3 install --upgrade pip' )
            coms.append( 'sudo pip3 install --upgrade PyOpenGL' )
        elif major_ver_ubuntu=="20":
            for pkg in pkg_list:
                coms.append( ( 'sudo apt install -y %s' % pkg) )

        for pkg in after_pkg_list:
            coms.append( ( 'sudo apt-get install -y %s' % pkg) )

        return self._doCommands( coms, self.debug )

    ########################################################
    def _doCommands(self, coms, debugFlag=False,ignoreErr=False):
        """
        Execute command list
        @param coms (list of string) commands to be executed
        @param debugFlag (bool) if True, do not touch files on disk
        @retval 0 No problem
        @retval -1 Error happened
        """
        if (debugFlag!=True) and (debugFlag!=False):
            msg = "doCommands: cannot invalid flag"
            print(msg)
            self._writeLog(msg)
            return -1

        for com in coms:
            com_sudo = ""
            if not debugFlag:
                
                ff = os.system(com)
                msg = "_doCommands::%s res=%d" % (com,ff)
                self._writeLog(msg)
                if ff!=0:
                    msg = "_doCommands::sudo %s" % com
                    self._writeLog(msg)
                    com_sudo = 'sudo ' + com
                    if os.system(com_sudo)!=0:
                        if not ignoreErr:
                            msg = "sudo fails: %s" % com_sudo
                            self._writeLog(msg)
                            return -1

            if com_sudo!="":
                self.logs.append(com_sudo)
            else:
                self.logs.append(com)
            
        return 0
    ########################################################
    def _writeLogs(self):
        """
        Save logs to file
        
        """
        fo = open("install_log.txt",'a')
        fo.write( "                            \n" )
        fo.write( "------- Write Logs ---------\n" )
        for log_line in self.logs[self.logs_index:]:
            dt=datetime.datetime.today()
            out_log = (self.date_fomat%(dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second))+log_line+"\n"
            fo.write( out_log )
        fo.write( "----------------------------\n" )
        fo.close()
        self.logs_index = len(self.logs)

    ########################################################
    def _writeLog(self,message):
        """
        """
        fo = open("install_log.txt","a")
        dt=datetime.datetime.today()
        df = (self.date_fomat%(dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second))
        fo.write( df+"Message >> "+message+"\n" )
        fo.close()

    ########################################################
    def Check_Scenario(self, scenario, pre_install_pack=[] ):
        """
        Scenario item = ( <source name>,[<required source name>,<...>],<source type> )
        <source name> : directory name like 'manyo-core', 'python-utsusemi'
        <source type> : 'manyo' or 'python' or 'python-utsusemi'
                        if manyo -> compile source
                        if python -> copy directory to /opt/mlfsoft/<source name>
                        if python-utsusemi -> copy direcotry to /opt/mlfsoft/python-utsusemi/
        """
        dicSe = pre_install_pack[:]
        self.InstallList = []
        for pname,depps,src_type in scenario:
            if pname in dicSe:
                pass
            else:
                dicSe.append( pname )
            for deppack in depps:
                if deppack in dicSe:
                    pass
                else:
                    msg = "Check_Scenario >> Senario is invalid. %s must be installed before %s."%(deppack, pname )
                    print(msg)
                    self._writeLog(msg)
                    return False
            if self._checkfiles(pname):
                self.InstallList.append( (pname, src_type) )
            else:
                msg = "Check_Scenario >> checkfiles is invalid : No source folder for (%s)."%(pname)
                print(msg)
                self._writeLog(msg)
                return False

        print( "self.InstallList = ",self.InstallList )
        return True
        
    ########################################################
    def Install_as_Scenario(self, scenario=[], pre_installed_pack=[] ):
        """
        
        """
        if len(scenario)!=0:
            if not self.Check_Scenario( scenario, pre_installed_pack ):
                return False
                
        elif len(self.InstallList)==0:
            return False
        
        
        self.InstallList.reverse()
        for dname, a_type in self.InstallList:
            if a_type=="libs":
                if dname == "lib-levmar":
                    if self.Install_Levmar(dname):
                        return False
        
        for dname, a_type in self.InstallList:
            if a_type=="manyo":
                do_make_file_path = os.path.join( self.cdir,"src",dname,"do_make_Utsusemi.py" )
                if os.path.exists( do_make_file_path):
                    ro = open( do_make_file_path, "r" )
                    do_make_coms = ro.readlines()
                    ro.close()
                    out_make_coms = []
                    for a_com in do_make_coms:
                        if a_com.find("CXX")==0:
                            out_make_coms.append( 'CXX            = "%s"\n'%(self.MakeDic["CXX"]) )
                        elif a_com.find("PYVER")==0:
                            out_make_coms.append( 'PYVER          = "%s"\n'%(self.MakeDic["PYVER"]) )
                        elif a_com.find("NUM_OF_THREADS")==0:
                            out_make_coms.append( 'NUM_OF_THREADS = %d\n'%(self.MakeDic["NUM_OF_THREADS"]) )
                        elif a_com.find("MANYO_PATH")==0:
                            out_make_coms.append( 'MANYO_PATH     = "%s"\n'%(os.path.join( self.PathDic["mlfsoft_root"],"manyo") ) )
                        elif a_com.find("isCompPara")==0:
                            out_make_coms.append( "isCompPara = False\n" )
                        else:
                            out_make_coms.append( a_com )
                    rw = open( os.path.join( self.cdir,"src","do_make_Utsusemi.py" ), "w" )
                    if rw==None:
                        return False
                    rw.writelines( out_make_coms )
                    rw.close()
        if os.path.exists( os.path.join( self.cdir,"src","do_make_Utsusemi.py" ) ):
            pass
        else:
            return False
        
        self.InstallList.reverse()
        coms = []
        for dname, a_type in self.InstallList:
            print( "## Install_as_Scenario step : %s, %s"%(dname, a_type) )
            if a_type=="manyo":
                coms.append( "mkdir -p %s"%(self.PathDic["mlfsoft_root"]) )
                coms.append( "chmod 777 -R %s"%(self.PathDic["mlfsoft_root"]) )
                coms.append( "cd %s; python3 ../do_make_Utsusemi.py install"%(os.path.join(self.cdir,"src",dname ) ) )
            elif a_type=="python":
                p1 = os.path.join( self.cdir,"src",dname )
                p2 = os.path.join( self.PathDic["mlfsoft_root"], dname )
                coms.append( "mkdir -p %s"%(p2) )
                coms.append( "cp -rf %s/* %s"%(p1,p2) )
            #elif a_type=="python-utsusemi":
            #    p1 = os.path.join( self.cdir,"src",dname )
            #    fname = dname[ (len(a_type)+1): ]
            #    p2 = os.path.join( self.PathDic["mlfsoft_root"],a_type,fname )
            #    coms.append( "mkdir -p %s"%(p2) )
            #    coms.append( "cp -rf %s/* %s"%(p1,p2) )
            elif a_type=="env":
                p1 = os.path.join( self.cdir,"src",dname )
                fname = dname[ (len(a_type)+1): ]
                coms.append( "cp -rf %s/* %s"%(p1,self.PathDic["userhome"]) )
        
        if self._doCommands( coms )<0:
            return False
            
        print(" ")
        print("-----------------------------------------------")
        print("  Install of ")
        for a_pack in self.InstallList:
            print("       - %s "%( a_pack[0] ))
        print("                                were completed.")
        print("-----------------------------------------------")
        print(" ")
        
        return True

    ########################################################
    def Install_Levmar(self,dname):
        levmar_path =os.path.join(self.cdir,"src",dname,"2.5" )
        manyo_lib_path = os.path.join(self.PathDic["mlfsoft_root"],"manyo","lib")
        manyo_inc_path = os.path.join(self.PathDic["mlfsoft_root"],"manyo","include")
        coms = []
        coms.append( "cd %s; make"%(levmar_path) )
        coms.append( "sudo mkdir -p %s"%(self.PathDic["mlfsoft_root"]) )
        coms.append( "sudo chmod 777 -R %s"%(self.PathDic["mlfsoft_root"]) )
        coms.append( "mkdir -p %s"%(manyo_lib_path) )
        coms.append( "mkdir -p %s"%(manyo_inc_path) )
        coms.append( "cp -rf %s/liblevmar.* %s "%(levmar_path,manyo_lib_path) )
        coms.append( "cp -rf %s/*.h %s "%(levmar_path,manyo_inc_path) )

        if self._doCommands( coms )<0:
            print("Installation of Levmar Library failed.")
            return False
        
    ########################################################
    def make_Utsusemi_Env(self):
        """
        
        """

        user_dirs = []
        user_dirs.append( os.path.join( self.PathDic["userhome"],"ana" ) )
        user_dirs.append( os.path.join( self.PathDic["userhome"],"ana","tmp" ) )
        user_dirs.append( os.path.join( self.PathDic["userhome"],"ana","xml" ) )
        user_dirs.append( os.path.join( self.PathDic["userhome"],"exp" ) )
        user_dirs.append( os.path.join( self.PathDic["userhome"],"exp","log" ) )
        user_dirs.append( os.path.join( self.PathDic["userhome"],"exp","xml" ) )
        
        coms = []
        for udir in user_dirs:
            if not os.path.exists( udir ):
                coms.append( 'mkdir ' + udir )
        
        coms.append( "mkdir -p %s"%(self.PathDic["mlfsoft_root"]) )
        coms.append( "sudo chmod 777 -R %s"%(self.PathDic["mlfsoft_root"]) )
        coms.append( 'mkdir -p %s'%(self.PathDic["data_root"]) )
        coms.append( 'sudo chmod 777 -R %s'%(self.PathDic["data_root"]) )
        
        ret = self._doCommands( coms )
        if ret==-1:
            msg =  "Error happends on executing command %s." % self.logs[-1]
            print(msg)
            self._writeLog(msg)
            self._writeLogs()
            raise UserWarning(msg)

        self._writeLogs()

        self._saveEnviron()
        
        print(" ")
        print("-------------------------------------------------------")
        print("  Utsusemi Base Environment Setting has been completed.")
        print("-------------------------------------------------------")
        print(" ")

        
    ########################################################
    def add_EnvironmaneVariables(self,BLcode):
        """
        """
        bash_file = os.path.join( self.PathDic["mlfsoft_root"],"python-utsusemi","utsusemi",BLcode,"bashrc.py3.%3s"%(BLcode) )
        if not os.path.exists(bash_file):
            msg = "Error: There is no bashrc template file (%s)\n" % (bash_file)
            print(msg)
            self._writeLog(msg)
            raise UserWarning(msg)
        
        fo = open( bash_file, "r" )
        bash_conts = fo.readlines()
        fo.close()
        
        rewrite_lines = []
        for a_line in bash_conts:
            if a_line[:3]=="XXX":
                rewrite_lines.append( "XXX=%s\n"%(BLcode) )
            elif a_line[:2]=="BL":
                pass
            elif a_line.find("MLFSOFT_PATH=")==0:
                rewrite_lines.append( "MLFSOFT_PATH=%s\n"%(self.PathDic["mlfsoft_root"]) )
            elif a_line[:25]=="UTSUSEMI_EXT_LIBRARY_PATH":
                rewrite_lines.append( "UTSUSEMI_EXT_LIBRARY_PATH=%s\n"%(self.PathDic["mlfsoft_root"]) )
            else:
                rewrite_lines.append(a_line)
        
        fw = open( bash_file, "w" )
        fw.writelines( rewrite_lines )
        fw.close()
        
        # add bashrc
        ss = '\n'
        ss+= '# For Utsusemi environment\n'
        ss+= 'if [ -f %s ]; then\n'%(bash_file)
        #ss+= '       source %s\n' %(bash_file)
        ss+= '       alias %s="source %s"\n' %(BLcode,bash_file)
        ss+= 'fi\n'
        
        try:
            fw = open( "add.bashrc", "w" )
        except:
            msg = "Cannot open add.bashrc."
            print(msg)
            self._writeLog(msg)
            return -1
        fw.write(ss)
        fw.close()
        
        coms = []
        coms.append('cat add.bashrc >> ~/.bashrc')
        coms.append('rm -rf add.bashrc')
        coms.append('mkdir -p %s'%(os.path.join( self.PathDic["data_root"],BLcode ) ) )
        coms.append('sudo chmod 777 -R %s'%(self.PathDic["data_root"]) )

        if (BLcode=="DNA"):
            coms.append("cp -rf src/environ-DNA/ana/* %s"%(os.path.join( self.PathDic["userhome"],"ana" )))

        ret = self._doCommands( coms, self.debug )
        if ret==-1:
            return -1
            
        ## End safely
        print(" ")
        print("----------------------------------------------------------")
        print("  Utsusemi Environment Setting for %3s has been completed."%(BLcode))
        print("----------------------------------------------------------")
        print(" ")
        return 0



PACK_DIC = { "MANYO-CORE":("manyo-core",[],"manyo"),
             "MANYO-UTSU":("manyo-utsusemi",["manyo-core"],"manyo"),
             "PYTHON-UTSU":("python-utsusemi",["manyo-utsusemi"],"python"),
             "PYTHON-UGAO":("python-ugao",["manyo-core"],"python"),
             "MANYO-TKT":("manyo-TKtools",["manyo-core"],"manyo"),
             "MANYO-SAS":("manyo-SAS",["manyo-utsusemi"],"manyo"),
             "PYTHON-EMAKI":("python-emaki",["python-utsusemi"],"python"),
             "MANYO-DNA":("manyo-DNA",["manyo-utsusemi"],"manyo"),
             "LIB-LEVMAR":("lib-levmar",[],"libs"),
             "MANYO-LEVMAR":("manyo-LevmarTools",["manyo-core","lib-levmar"],"manyo"),
             "ENVIRON-DNA":("environ-DNA",["manyo-DNA"],"env")
             }

DEF_PACK_ORDER = ["MANYO-CORE","MANYO-UTSU","MANYO-TKT","MANYO-SAS","PYTHON-UTSU","PYTHON-UGAO",
                  "PYTHON-EMAKI","LIB-LEVMAR","MANYO-LEVMAR","MANYO-DNA","ENVIRON-DNA" ]

def UtsusemiDefaultInstall(num_of_cores=8):
    
    ITU=InstallToUbuntu()
    ITU.installUbuntuPackages()
    ITU.MakeDic["NUM_OF_THREADS"] = num_of_cores
    install_scenario = []
    for a_key in DEF_PACK_ORDER:
        install_scenario.append( PACK_DIC[a_key] )
    
    if ITU.Check_Scenario( install_scenario ):
        ret = ITU.Install_as_Scenario( install_scenario )
        if (ret):
            ITU.make_Utsusemi_Env()
            ITU.add_EnvironmaneVariables("SIK")
            ITU.add_EnvironmaneVariables("AMR")
            ITU.add_EnvironmaneVariables("HPN")
            ITU.add_EnvironmaneVariables("SAS")
            ITU.add_EnvironmaneVariables("TKtools")
            ITU.add_EnvironmaneVariables("DNA")
            print("  Utsusemi Default Install was completed.")
    



if __name__== '__main__':
    if sys.version.split(" ")[0]<"3.0.0":
        print( "ERROR : This script requires Python 3.x")
        print( "        Your python version is %s"%(sys.version.split(" ")[0]))
        sys.exit(-1)
    
    if (len(sys.argv)==2) and (sys.argv[1]=="-all"):
        UtsusemiDefaultInstall()
    else:
        ITU=InstallToUbuntu()
        print("=================================================================")
        print("       Welcome to Utsusemi software installer (ver.%s) " % (__version__))
        print("=================================================================")
        menu_title = " in common directory "
        continue_flag = True
        while( continue_flag ):
            force_return_flag = False
            ITU._showEnviron()
            print("""

-----------------------------------------------------------------
       Utsusemi Install Main Menu for Debian or Ubuntu
-----------------------------------------------------------------
  
  0... Install ALL modules for an instrument
         note: this will DELETE all of modules installed before.

  ----------------------------------------------------
  1... Required Ubuntu Packages
  2... Utsusemi software (Manyo-lib and common code)
  3... Utsusemi software for each BL

  7... Change Install Directories
  
  9... Uninstall

others.. exit
-----------------------------------------------------------------""")

            ret=input(">> Which ? ")
            if not ret in ["0","1","2","3","7","9"]:
                print("Exit without installation.")
                continue_flag = False
                force_return_flag = True
            else:
                choice = int(ret)
                
                if choice==9:
                    print(" ")
                    print("Do you want to delete Raw data or xml data, too ?")
                    ret=input(">> answer ?[y/N] ")
                    if ret=="Y" or ret=="y":
                        ret = ITU.uninstall(True)
                    else:
                        ret = ITU.uninstall()
                    if ret==-1:
                        print("Failed to uninstall.")
                        sys.exit()
                    print("Uninstall is finished.")
                    print(" ")

                elif choice==7:
                    while(True):
                        print("""  ------------------------------------------------
  Install Directories Menu
  ----------------
  1..... Change Directories to private area
  2..... Return Directories to MLF default
  3..... Load Previous Install Directories
  4..... Show Present Setting
  
  0..... Return to Main menu
""")
                        ret1 = input(">> Which ? ")
                        if not ret1 in ["0","1","2","3","4"]:
                            continue
                        if ret1=="0":
                            force_return_flag = True
                            break
                        elif ret1=="1":
                            ret = ITU.AskPrivateInstallPath()
                            if ret!=0:
                                print("Something wrong")
                            menu_title = " in private directory "
                        elif ret1=="2":
                            ITU._initialEnviron()
                        elif ret1=="3":
                            ITU._readEnviron()
                            ITU._showEnviron()
                        else:
                            ITU._showEnviron()


                elif choice==1:
                    ITU.installUbuntuPackages()

                elif choice==2:
                    print("For compiling Manyo-lib, please select the number of threads.")
                    ret=input(">> How many threads? [>1, default=1] ")
                    try:
                        nc = int(ret)
                    except:
                        nc = 8
                    ITU.MakeDic["NUM_OF_THREADS"] = nc
                    install_scenario = []
                    install_scenario.append( PACK_DIC["MANYO-CORE"] )
                    install_scenario.append( PACK_DIC["MANYO-UTSU"] )
                    install_scenario.append( PACK_DIC["PYTHON-UTSU"] )
                    install_scenario.append( PACK_DIC["PYTHON-UGAO"] )
                    install_scenario.append( PACK_DIC["PYTHON-EMAKI"] )

                    if ITU.Check_Scenario( install_scenario ):
                        ret = ITU.Install_as_Scenario( install_scenario )
                        if (ret):
                            pass
                        else:
                            print(" ")
                            print("!!!!!!!!!  Install Failed !!!!!!!!!")
                            print("     Utsusemi is NOT installed.")
                            print("-----------------------------------")
                            print(" Please send install_log.txt to ")
                            print("  your contact person.")

                elif choice==3:
                    print(" ")
                    ret = input(">> Have you installed ManyoLib and Utsusemi Base system ( menu number 2 ) [y/N]? ")
                    if ret!="Y" and ret!="y":
                        pass
                    else:
                        print("---------------------------------")
                        print("  Select Package you install")
                        print("---------------------------------")
                        print(" [ 1] BL01 4SEASONS (SIK)")
                        print(" [ 2] BL02 DNA (DNA)")
                        print(" [14] BL14 AMATERAS (AMR)")
                        print(" [11] BL11 PLANET (HPN)")
                        print(" [15] BL15 Taikan (SAS)")
                        print(" [TK] TKtools")
                        print(" [-1] Exit menu")
                        print("---------------------------------")
                        print(" ")

                        pre_installed_pack=["manyo-core","manyo-utsusemi","python-utsusemi"]
                        while(True):
                            install_scenario = []
                            ret_BL=input(">> Which Package? ")
                            ret_BL = ret_BL.strip().upper()
                            BL_name=""
                            if ret_BL=="1":
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-SIK"] )
                                BL_Name="SIK"
                                break
                            if ret_BL=="2":
                                install_scenario.append( PACK_DIC["LIB-LEVMAR"] )
                                install_scenario.append( PACK_DIC["MANYO-LEVMAR"] )
                                install_scenario.append( PACK_DIC["MANYO-DNA"] )
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-DNA"] )
                                install_scenario.append( PACK_DIC["ENVIRON-DNA"] )
                                BL_Name="DNA"
                                break
                            elif ret_BL=="14":
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-AMR"] )
                                BL_Name="AMR"
                                break
                            elif ret_BL=="11":
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-HPN"] )
                                BL_Name="HPN"
                                break
                            elif ret_BL=="15":
                                install_scenario.append( PACK_DIC["MANYO-SAS"] )
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-SAS"] )
                                BL_Name="SAS"
                                break
                            elif ret_BL=="TK":
                                install_scenario.append( PACK_DIC["MANYO-TKT"] )
                                #install_scenario.append( PACK_DIC["PYTHON-UTSU-TKT"] )
                                BL_Name="TKtools"
                                break
                            elif ret_BL=="-1":
                                break
                            else:
                                print(' "%s" is Invalid value'%(ret))

                        if ret_BL!="-1":
                            ITU.make_Utsusemi_Env()
                        if len(install_scenario)!=0:
                            if ITU.Check_Scenario( install_scenario,pre_installed_pack ):
                                ret = ITU.Install_as_Scenario( install_scenario, pre_installed_pack )
                                if not ret:
                                    print(" ")
                                    print("!!!!!!!!!  Install Failed !!!!!!!!!")
                                    print("     Utsusemi is NOT installed.")
                                    print("-----------------------------------")
                                    print(" Please send install_log.txt to ")
                                    print("  your contact person.")
                            else:
                                print("Install Scenario is invalid.")
                        
                        ITU.add_EnvironmaneVariables(BL_Name)

                elif choice==0:
                    print("For compiling Manyo-lib, please select the number of threads.")
                    ret=input(">> How many threads? [>1, default=1] ")
                    try:
                        nc = int(ret)
                    except:
                        nc = 8
                    UtsusemiDefaultInstall(nc)

                else:
                    print(" Invalid input ")

            if force_return_flag:
                pass
            else:
                ret=input(">> Back to Main Menu? [Y/n] ")
                if ret=="N" or ret=="n":
                    continue_flag = False
                else:
                    continue_flag = True
                        
                        



# -*- coding: utf-8 -*-

# Resource object code
#
# Created: 水 6月 5 17:02:10 2019
#      by: The Resource Compiler for PySide (Qt v4.8.7)
#
# WARNING! All changes made in this file will be lost!

import sys
if '--pyside1' in sys.argv:
    from PySide import QtCore
else:
    try:
        from PySide6 import QtCore
    except:
        try:
            from PySide2 import QtCore
        except:
            from PySide import QtCore

qt_resource_data = b"\x00\x00\x1a\xec\x89PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\x00\x00\x87\x00\x00\x00\x87\x08\x02\x00\x00\x00\xb3\x85\xdd]\x00\x00\x00\x06tRNS\x00\xff\x00\xff\x00\xff7X\x1b}\x00\x00\x00\x06bKGD\x00\xff\x00\xff\x00\xff\xa0\xbd\xa7\x93\x00\x00\x00\x09pHYs\x00\x00\x12t\x00\x00\x12t\x01\xdef\x1fx\x00\x00\x1azIDATx\x9c\xed][T[\xd7\x99\xfe8:\xba\x22\x09\x89\x8b\xc0\x02\xd9\xb2!\x868`\x13\x1c\xe3KM\x1c\x974$\xcd\xc53u:\x9d\xac5v\xdb\xd5\xf6!\x9d\x07\xf7a\xfa\x92\xbeL\x1f&/\x9d\xb5\xa6~i\x1f&]]\xd3<\xb8i\xe2\xaeq\x12\x8f\xeb\xb44up\x8d/1\xc61v\x80\x80\x91-\x10 \x03B\x12H\xe8v4\x0f\x87\x8b\x10\xe7\xb2\xf79G@:\xf3=\x19\x9d}\xf6>>\xdf\xf9/\xfb\xdf\xff\xfewQ6\x9b\xc5\x97\x17\xb1i\xd1Kf'\x8a\x98u|\x14-\xc1n\xf4\x03\x10 \xcb!\x1eB*\x86t\x0c\x89\x082\x09\xc4C\x14\xb7\x1bK\xc0\x1aar@g\x84\xc1\x06\x93\x1d\x8c\xbe`\xcf\xaa\x0d\x8a6\xa9\xacd9\xcc\x07\x11{\x84\xd8\x14\x92s\x1aw\xae\xb7\xc0R\x06K\x05\x8a]\x9b\x93\xa1M\xc6J\x22\x8c\xf9 \xe6'\xe9\xa4A\x0d\x8c%(v\xa1\xb8\x12f\xe7:\x8dH\x80\xcd\xc1J:\x81\xe8\x18\xc2\x0f\xb4\x17\x0br\xe8-\xb0\xd7\xc0\xee\x81\xde\xb2a\xcf\xb0\x84\x8df%:\x86\xb9ID\xc76\xf2\x19\xf2P\xec\x82\xd5\x0d{\xf5\x06:\x0b\x1b\xc4J\x96\xc3\xac\x0f\xa1!\xa4\x13\x1b0:\x09\x18=\x9c\xdb\xe1\xdc\xb1!\x86g\xddY\xe1R\x98\xf5avd\xf3\xf2\x91\x0bF\x0f\x87\x17\x8e\xed`\x8d\xeb9\xec:\xb2\xc2\xa5\x10\xba\x8f\xd0\x08\xb8\xd4:\x8d\xa8\x15\x8a\x188\xbcp\xd6\xad\x1b7\xeb\xc5J\xd8\x8f\xe9\xcf\xbf\x1c\xf2!\x06F\x8f\xd2Z\x94>\xb6\x0eC\x15\x9e\x95\xe4\x1c\x1e\xdd\xc5|\xb0\xb0\xa3\xac\x1b\x8c%p5\x15\xda\x8d.$+Y\x0e\xa1aL\x0f\x22\xcb\x15j\x88\x8dB\x89\x07\x15O\x14\xce\x11(\x18+\x890\xc6{6r\xfeQh0zT5\xc3ZU\x88\xbe\x0b\xc3\xca\xac\x0f\x8f\xee\xfe\x0d\x8a\xc8Z8w\xa0\xe2\x09\xcd{\xd5\x9a\x95,\x87\xe0g\x08\xfb\xb5\xecs\x93\xc3\xec\xc4\x96}\xda\xbag\x9a\xb2\x92\x9cC\xe0\xc6Fi\xad\xc0\xe8\x9dK\x17NG#\x8bn\x85\xc1T\x5cYU\xef\xdaR_\xe3m\xa9t\xd7\x17vlF\x8f--(vi\xd5\x9fv\xac\xcc\x07\x11\xb8\xb1\x81Z\xeb\xd2\xc5\xd3\x03}\x9d\x82\x97,\x16\xa7w\xe7\x81\x1d\xf5\x87\xdd5M\x05|\x82\xf2\x06\xad\xfcf\x8d\xd6W\xc2~\x04?+\x04%\xb1X\xc87xu|\xac/>?\x0b`O\xebq\x8f\xb7EA'\xf7z/\xdc\xeb\xbd`\xb3\xbb\xf6\xec;^\xdf\xd8\xaec\x0d\x9a?*\xa6\xfa\xc1eP\xde\xa0\xbe'-d%\xec\xc7d\xaf\xfaG\xc9\xc3d`\xa0\xef\xd6\xfb\xc3\xfd]\xb9?\xd67\xb6\x1f\xe98%\xd8~\xa0\xaf\xf3\xd2\xc5\xd3$=\x1bL\xc5MO\xbe\xd2\xd4\xf2\x8a\xc1T\xac\xc1\x83\xe6\xa1\xc4\x83\xcaf\x95}\xa8fez\x00\xd3\x83*\x1f\x22\x0f\xd1H\xf0z\xd7\x7f\xe5\xf1\xc1\xe3p\xfb\xeb\xbb\x9a_\x10\xbb\xf1\xec\xdb?\x9a\x0e\xde'\x1c\xc5`*\xde{\xf0\xb5]\xbb\x9f\xd7^n\xacU\xd8\xb2WM\xc8Y\x1d+ZS\x92\x5c\x98\xef\xbdq\xf6N\xcf\xfb\x99tr\xed\xd5Jw\xfd\xb1\xd7~&q\xbb\xdf\xd7s\xe1\xec\xbfR\x8dh\xb18\x8f\xbcpJ\x81V\x94A\xb1\x0b\xee}\x8a\x89Q\xc1\xca\xcc\x17\x98\xeaWx\xaf\x10\xee\xf4\xbc\x7f\xb3\xfbLra^\xf0\xaa\x8e5\x1c?\xf1sGi\x8dt'\x9f\xfe\xf57=W\xdf\xa3\x1d\xda\xe3m9\xf2\xc2)\x8bE\xd38J\xb1\x0b\xd5\xfb\x95\xdd\xaa\x94\x15MmI,\x16\xfa\xe3\xb97'\x03\x03\x12m\x8et\x9c\xaaol'\xe9\xed\xd3\xab\xff\xd1w\xf3\xaf\xc9\x05\x01i\x93\x80\xc1T\xdc\xfa\x95\x93\x12\xeaQ\x09\x94\xda\x18E\xach\xea\x04\x0f\xf4uv_zKLDx\xb4\xb6\x9dln}\x95\xb0\xc3L:\xe9\x1f\x7f\xfb\xe6'\x97\xa7'\xc4\xf3\x92D`\xb3\xbbZ\xdb\xbe]\xdb\xd0F{\xa3(\xcav\xa2\x8cz\xb6D\xcfJ\x22\x0c\x7f\xb7&k$\xb1X\xe8\xd2\x85\xd3~_\x8ft\xb3\xa6\x96W\x0e\x1e\xfd>U\xcf\x99\xf4\xc2T\xf4\xbd/n\xdf\xb9w\xf3s\x05\x0fV\xe6\xda\xd1\xdavR3c\xe3j\x82\xc3Ku\x07%+\xe9\x04\xfc\x97\x91\x8aQ\x8d!\x88\xe1\xfe\xae\xae\xce_H\x8b\x08$]aid2\xf3\xb1\xec\x85\xf1\x07\x0f\xbb>\xec\xca\xa43\x0azhjy\xa5\xb5\xed\xa46\x1e\x9a{\x1fU\x1c\x93\x92\x95\xd1n\xc4\xa6\xa8\x9fi\x0dn^9s\xb3\xfb\x8cl3\xc5\x94,!\x15\xcb\xfeO2\x19\xfd\xe8\xb7\x1f\xcdN\xcd*\xb8\xdfQZ\xf3\xdc\xb17d]\x0cy0zl{\x9a<{\x86\x86\x15-\xfc\xe0L:\xf9\x97\x8b\xa7\x05\xe7\x22yPM\xc9\xe2\x80s\xa9?YKS\xbd\x97{{/\xf7\xd2\xba\x00\x00t\xac\xe1\xe0\x91\xefi\xe0\x05\x98\x9d\xa89D\xe8+\x13\xb3\x12\x9b\xc6\xe8\x15U\x8f\x05D#\xc1\x8f\xce\xbdI2\xd1\xdb\xd5\xfc\xc2\xe1\xf6\xd7U\x0e\xb7\x8cx\xfa\x96\xd99\x16\x9b\x8bu\xff\xa1{\xb8oXA\x0f\xcd\xad\xaf\xb6\xb6\x9dT\xfb\x1c\xc4a\x7f2V\xd2\x09<\xbc\xa4r\xd5}20\xf0\xc7so\xc6b2I\x91\x05\x9a\xd6q\xd9`\xc6xUod\x02\x0f\xc6/\x7f\xd0\xa5@\xa1i#\xbbd\x06\x86\x8c\x15\xd5\xe6\xc4\xef\xeb\xf9\xe8\xdc\x9b\x823\xf6\x5cx\xbc-\xed/\xfe\xb8 \xe1)\x00L\x22\xa5\xbb\xac7\xc5\x01\xf4^\xee\xed\xbb\xda\x17\x9b\xa3s[<\xde\x96\xe7\x8e\xbd\xa1\xca\xfe3zx\x8f\xca.\xc6\x10\xb0\xa2z\xc2HN\x89\xda\xff3\x09\x0c#\x19\xdd]\x1d\x8bL:s\xef\xd3{\xb7/\xdf\xa6\xe2F\x83\x87\xb4Uc\x8b\x8c&\x90c\x85K\xc1\xf7\xb1\x1a\xddEHImC\xdb3\x1d\xa7\x0aN\x09\x0f&\xc1\xe9o1\x86)@\x097\xb5\x0dm\xed/\xfeX\xd5\x03T\xef\x97^\x22\x93c%x\x07\xb3>\xc5\xa3\x93Pb0\x15\x1f<\xf2}\xc2h\x8a\x96`\x1fe\x0d=E\xba\x14\x96\xb8\xe9\xbb\xda\x17\x9d\x8d\x92\xdc\xaa\xd6\xf8\x1b\xac\xd8vD\xc2\x1f\x93d%\x11\xc6\x83O\x14\x0f=;3z\xf6\xed\x1fISR\x90\xb0 \x05R0\x8cd\xf5\xc3E\xcc\xe24\xd3?\xe4\x1f\xec\x1d\xf4\xf5\xfbd'\x9e\xcf\x1d{\xc3[w@\xf9\xc8\x92\x91\x18IV\x1e|\x82DX\xd9\xa0\xc9\x85\xf9\xff>\xf3\xe3\xd9\x99Q\xb1\x06:\xd6p\xb8\xfduU\x22\xe2{\x84@\x08\x8f\x22H\xa4\x17\x7f)1\xa3\xc2\x8e\xba*\xd8\xcd4\x1d\xe5s\x93\x5cH\x0e\xdd\x1d\xfe\xfc\xc6=\x89H\x9a\xa3\xb4\xe6\xf8\x89\x9f+W\xb9E\x0c\xb6\x1d\x81\xc1*|Q\x94\x95Y\x1f\x82w\x14\x0e\x09t\x7f\xfc\xd6\x9d\x9e\xf7\xc5\xae\xba=M\x87\x9f}]\xf9\x9c\xb9g\x04\xd7\x86\x10\x13\xb7v\x1d{\xd0\xe8\xa1\xec4\x05\xc3\x08\xa7\x1fb\x98\x95\xa8kt6\xea\x1f\x1e\xbd\x7fg8\xe0\x0b\xac\xbdAz\x09N\x1e\xd6*\xb8\xf7\x09^\x11_\xb7\x9fQ>\x8d\x9f\x9d\x19\x15\xa3\xc4\xedij9\xf4\x8f\xaa\xb2\x1a\xfa\x03\xf8\xf8\xaeL\x9b\xce>\xecp\xc1B\x95\x0d\xa4Gr'\x93\xac\x05;\x99\xc2\x17zs\x14\x80\xcda\xdb\xb5\xf7\xf1]{\x1fO.$\x03\xbe\xc0\xb8o<\xe0\x0b,\x0bP<\xa6$\x8a\xb3\x82\xb9\x09$\xc20\x96\xac\xbd\x22\xc2\xca\xdc\x84\x1a\xbf\xcb7tu\xed\x8f\x1a\xf0\xc1\xe3\x06\xc1\xe4<\x9d\xc1\xb5!\x1cU\x90?\xa7C\xda\xad\x87\x1b\xf3\x09\x8e\xf1e\x8a\x86\xf5&\x0e\x80\xc1d\xf06x\xbd\x0d^\x00\x99tfr,\x18\xf4O\xdam\xc2\xfa\x87\x02\xd3\x83\x82\xe2\x22\xc2\x8a\xbax\x979\xc7z\xf3Y?\x8f=\xfeUm\x92\xb2\x82a\x04\xc9L\xddg\x0f\xd1\xd6\x00V\xa7p \xce\xc8p\xf5\x0c\xea\x91\x89\x83\x99\x8a\xa7Ft\x86\x88\xc1\x04\x00:V\xe7\xde\xb6\xc5\xbdm\x0b\xe2\x8dH\xcb\xf5#\x8d\xf9 \xd2\x89\xb5\x93J!\xbb27\x81\xc0\x0du\xa3!\x1a\x09F#\x936{\xa5\xcd\xaeY\xee\x1a\x00\x5c\x19D7\xf1\x17\xf3\xad\x83\xa8)\xd3rt\x00L\x08\xbc\xe1\xe1,\xe0\xa8|\x0a\x11\x08\x05\xc7\x84dE\x8b\x04\x09\x9b\xdd\xa51\x1f\x0a\x10\x08\x11\xb1\x12\x08\x81\xe3`\xb7\x10yn\x9c\x13\xdaf\xbd\xcd\xfa\xd6\xeeWZ\xc3JlZ\xb17\xbc\x1e\xc8\xd0\xbc\x92\xb4d\xe3`\x18\x7f\xf9\x1c\xfe\x9c\xf8^\xb3\x17_\xa9\x87i}7Bf9\x84}ys\x975\xac\xcc\x09\xb8\x80\x9b\x08\xc5Z$Y\xa73\xb8\xf49z}\xf9\xbf\xf7\xfa\x10\x08\xe1\x9b\x07\xd6\x9b\x98\xc8h\x1e+\xab'\xfdY\x0e\x91\xcd\xb4\xc9z-Ji\xdc\x1e\x8b\xd0\x14/\x9d\xc1\xb9O\x05(\xe1\x11\x0c\xe3\xfd\x9bP\xb4\xa2\xac\x1c\xa9X^A\x9a\xd5\xac\xcc\x07\xd7{+i$\x8e\xd1i\x8a\xb7P\xe5 m\xc9\xea\xb0sK\xfe\x8f<%\xbeGR7\xfa\xa7\xd0\xfd\x05\xe9(\xab\x91I'\x03\xa3wd\x93\x11\x04\x10]\xb5\xb7d\xb5\x06\x8b\xae\xa3\xfaZH\xa1{\x10=#\x00`\xd2\xa3\xa1\x1a\xfbj\xe5\xed\xadI\x0fO\xf9*c \x86F\x8f\xc0,\xb2\xab_\x86\x12\x1e=#\xd8\xbb\x9dr\x12\x8a\xe4\xc2\xfc\xd9\xb7OE#A\x83\xa9\xf8\xf8\x89\xd3t\xceNt\x02.n9^\x99#+\x5c\x0as\xe3T\xcf\xa1\x1c\x918\xce\xfcu\x91\x12\x00\x0b)\xf4\xfa\xf0\xeb\xbf\xe0\xfa\x90\xfc\xbd5\xa5DCT\xae\x993\xf7\x07VF\x94F:\xa3@\x5c\xba:\x7f\xc1\xef\x9eI.\xcc\xff\xf9\xbcT\xe6\xad\x00V\xbf\xfc\x1cV\x22c\xeb\xb4\xfb$\x12\xc7;W0\xb3f\xf3Q:\x83\xae~\x9c\xbd&\xa3\xd0\x9a\xb7\x11\xcd\x0d\xb7\x96\xaf\xfa3\x96@'MX\xaf\xd7G:]\x05\x00L\x07\xef\xe7\xa6\x88L\x06\x06\x04\x03\x1cR\xc8QT9\xac\xc4\x08D[=\x16Rx\xe7\x0a\x22q\xd1\x06\xbeG\xb8x[\xaa\x07\x8b\x11\xbb\xb7\xca\x8cb7\xe7+\xc3\x0b\xbdX\xa04\x99C\x93\xe4m\xfbn}\x90\xf7\xcb\xed\x1bg\xe9\x86\xcb1\xf8\x8c\xe0\xaf\x04\xc8\xa4\x92\xa1\xc0\xe8\x1d\xd9E\xc6|\x5c\xbc-E\x09\x8f\xfe\x80\xa8\x8f\xc4c\x7f\x9d\x8c\xb8\xe4\x09J\x7f\x80\xc8\x9c\xe4!*\xf7\x9cK\xc8\xa4\x93#k$c20\xb0\xbc\x1d\x90\x08\x5cjy\xa6\xb8\xc4J\x22L\xe1}1a\xae\xf8\xe2\x1f\xde\x7f\xe3\xc3w~\xf2\xbb_\xff\x90\xc2\xe5\x18\x9a\xc0\xd0\x04Q\xcb\xee/\xa4\xf4\x98\xac\xb8T\xe7\xd8\x9etF>\xc6,6\x0a\x19\x02\xa3}\x82/a\xb8\x9fr\xcdpI0\x98\xbc\xbf\xe5\xc1$\xb2\x96\xeeX$2\xfe`\x0c@4\x12\xec\xea\xfc\x05\xe9\xbd\xe4&4\x96\x90\xb1\xcc\xfb\xebD\xe7z\xac\x0eu9\xd9=\x9f=\x94Z\x89\x11\x03\xab\xc3\xde\xed\x84m\xef\x0f\x08'\x1d\x8e\xfan\xd1\x0d\x1a\xcfc%N\xcc\x8a~\xa8\xa8(\xfdpx\xc5\xbf\xf6\x0d]\x93\xcd\xf2\x02\x80\xd1i*\xfb)\xe3\xc5Z\x8c8\xb2K\xf8R]\xe5\x0aa|H_\x01vo%\x97\x95DR\xd8w\x9d\x1c\x1f\xa0\xd3\xf0Je%\x03\xfd\x03\x00\xf7\xef\xac,rd\xd2I\x92\x0cU<\xa4\xde\xb7\x80\xf3=R\x9fy\xa3g\x95L,\xe3\x89\x9cU\xc8\xd1\x19%\x82\xe2v\xa2\x8dt\xd3ira\xdeV*l\xe42\xe9\xe4\xe4\x84\xd4\xa6\x9c|,\x99\x16\x06\x00R1R\xa3\xc2\x06P\xc4%\x17\x92\x93\xa3\xab\xfc\x93\x07C\xd7\xe4\xefU\x10\x5cZH\xe1CI%\xf0\xe2\x93p\xad\x9e\x97\xb8J\xe0\xadX\xf9s\x84\xbe\xaa\x8f\xab\x04\x7f\xdfJ\xbe035}K'\xdex:H6CZFr\x0e\x8b\xac\x90\xd7-0\x8c\x01\x08\xf8\x02y) D\xa2J\xa3\x13V\xe0\x9fB\x9fx\xa5\x0bV\x87o\x1e\x80\xa7|\xe5\xcf\x8e\xdd\xab\x1a\xac\x9d\x15I\xc0\xa4G[\x03N\xb4Q}@E\xfa\x90D.Y8D\x19WL\xcea1\xe2B\xcaJ\x06\xba\x19\x00\xe3\xbe|5\xca\x8b\xaa\xcc\xea/\xab\xc3\xfe:%\xeePW?\x1a\xdc\xa2\x1f\xafI\x8f\x7f8\x80\xa1\x09\x04#\xa8\xab\xcc\x17\x1d\x124z`3ck\x19J\xad\x0a\xbe\x1b\x83eA\xe2*\xf9\x9e\xe6E$\xe7\xb1\xc8J\x9a\xcc1gg\x01\x0e\x80`\xc2\xc7tpD~M\xbee;\x1eE\xa4\xbe}A\xc4\x12\xf8\xec!Z$=\xa2\xba*a\x1bS[)\xea2x\xca\xb1\xb3\x0a\x0d\xd5*\xe3\xf6\xf62\xbdD\xf2XH<\xf7J\x18\xd4\xb2\xa2[\xfc\xef\x09&\xb4\x93\x8aj\xc7\x1e\x00\xd4\xc4\xdc~ \xc5\xca\xcc\x1cb\x090\x0c\xdck\xb2\xfd\x9a\xbdp;14\x89\xd1\x19\x00(6\xc0i\xc5\xd62T9\x94/\xe9\xe7\x82\x09\xeb\x0dl|N\xf4\xb3N.\xcc\xc7b!\x8a4\xc4d\x14t\xac0a\x00\xb3S\xb3\x82\x9f\xc6\xec4\xf1G\xd1\xb1\x07\xd5\xa5\xb8t\x8f\x22\x0423\x87H<?\x88\xc2/^\xf5\xf9W\xcd7=\xe58\xf4\xd8\xaa\x85aW\x89\x12\xb5F\x08F~\x06\x9dI\xd3Dz\xb2\x1c\xd2\x09\x06Y\x8et\x9f#\x13\x01 \x96N\x18\x0dSD\x8d\xd0\xe8\xc1\x89\xa7\x85u\x8e\x18\x22\xab\x1f2\x18\xc6\xdb]\xe8\xf5\xe5\x87\x00\xfcSx\xa7\x1b\xe7>\xa5\x8ez)B2#\x1fe\x8f\xcdQ\xd61O\xce\xb1HK\x19\xab\x1c\xa4\xc0$ \xc1\x0aU\xcc\x07\x80\xdd\x8ccO!\x12G\xf7 \x86&\xe4_b\xae\x1dNgp\xfe\x96\x94\x7f54\x81D\x1a\xdf\xd8\xa7\x8d\x9a\x12G\x16\xf2\x1ft\x86\xa3\xde\xf6G\x5c\xd2\x82Y\x9c\x8bI\xec\x92\xa2&\x06\x80\xdd\x8c\x8e=\xf8\xe7\x0e\x1c{\x0a\x8d\x1eQ\xc3\xdb\xec]\xb56\xfc\xd9Cy\x97\xd7?\x85.-Kg\x08\x82a\xe5Y\xa1^\x9a\xcc,\xb0H\x919`K\xacH$D\xc7\xe6B\xca\xb3\x8dx'\xaac\x0f\x82a\x04#\x08\xcdc|\x16\x00J\xcc\xa8.\xcdO\x1a&\x0c\xa2\xa8L\xd4#\x00k\x90\xd7\x93\x09ZV8\x8e\xbc>Xli\x0c\xd1\x00\x86\x02Q\x15\x80\xacq\x8e%H\x83(\xe9\x0c&f\xb5O\xd4[\x063G\xb2%8\x99\xa4e%\xc5\x22CfW\x98\xc5\xdcM\x05\x9b\xa35F0B\xd1X:Qotz\xa5Y\x9a\xc3x\x08\x99,\xb68Pl$\xdan\xc1\x10\xf9\xae\xd4\x1a\x8cK\xb1\xe0\xc8V\x85\x8b\x12\x00\xa4\xb7BEg\x83P]/@\x1e\xb34\xff\xc9\xbcD\xbdt\x06\xfd\x01\x8c\xcd\xe0~PT\xe0\xf8T\x8d\x8f\xef\xc2\xedD\xc7\x1e\xc9\x5c\xa7E\xf5e\xb6j\x91\xda\x9a\x03\x16Y\xb2\xac\x1f&\x81\xcd (\x00\x1c4;\x8c\x97\xbfw>A\xa2g\x84\x22\xcb)\x10\xc2\xdb]ho\x94\xdd\x0a#\x11\x9dT\x06\x06E\x84=.\x00H$\xa4X\xa1^-V\x06\xf2\x940\xe4,\x15\xff\xfe\x06\xae\x0fQ\xa7\xdf\xa53\xe8\xec\x13\xf5\xda\x19\xd2%dZ\x10{\xc6EY\x00s!)\x0d\xa6v\x97\x0d!Lz\xd2\xb9\xba\xdb\xb9(+C\x13D)d\x82\xe0]\x06I\xb0:\xa9/[\xc7R\xc7\xd9\x98\xcdp`\x92\x0cb\x09\x8cNctz%\x0d\xa3c\xb7\xbc\xbf\xcb\xcfRy\xa8\xcc\x1b\x9e\x13q\x88\x8a\x96\xec\x8aM\xea\x1d\x9ai\xf7\xe2\xb2\x96M|2\xe1\xcc\x1cn?\xc0\xd0\xc4\x0a\x19\xacnQ\xcb\xbbJ\xb0\xafVj#\x8b\xc5\x88cO\xad\x84\x03\xaa\x1cp\x95\xd0\xadO\xe7\xc2!\xf2\xd2\x97\xfcRiVX\xfa\x1d\xad\xec:\x9f\xc2C\x84>?n\x09%\xc9\xf1Z\x9e\xdf\xf0xh'\xdcN\x5c\xe8\xcdw\xa5X\x1dZ\xb6c_\xed*\xf9\xe0\xe9|\xf7\xaa\xc2\xb4n\x83\xd8\xb7\xbb(\xaf\x96b)\x1f\xccl\xa51\x84\x00X#KUGTsgc\x15\x02!\xcc\xcc\xa1{P*a,\x9d\xc1\xfd\xe0\xa2S\xe4\xad\xc0w\x9f\xc1Tdq\xb6\x01`k\x99\xe8\xe6 \xb7\x13\xc7[\x15F-M\x22\x1f{\xc6\xc0\xc7\xdc\x0dfM\xbf\xec\x22\x1d\x0b\x1d\x85\xce\x95\x16U\x85\x98\x99\xc3\xc7w\x95d\xd1\x010\xe9QSF:{\xaf)\xc37\x0f\xe0\xdd\xabt\xc4\x98\xf4\xe2\xd3\xc9\xc5\x0f\xdaV\x22\xb5\x7f\xc3f\xaf\xa4\x18\x0e\x80\x8ee\xc0\xe8\xb5RbJj\x15]\x1f\xc2\xdb]t\x94\xb8\xecRW\x87&\xd03\x22\xfa\xde]%8\xf1\xf4\xaat\x0bYH8\xe2\xdc\xe2{3\x9a4\x95\x15\x83\x8d\x01\xa0\x95\x1bf0P\xb2r}\x08]\xfdt\xba\xdeb\x94r\x8b{}8\xf7)>\xbe\x8b_\xfd\x19\xfd\x22\xbb>\xecf\x1c\xdf\x8f\x8e=\xa4\x8eY\xad\xfc\x97n0\x19,V\xe1w\xa8c\x0dt\x11[\xd6\x88\x22\x86\x01 \xb8\x11?\x1fY\xad\xab\x11\xf9\x1e)\x89\xb4\xef\xd9&uu97s!\x85\xf3=\xb8\x22\xee\xa45z\xf0\xddg\xd0\xec\x95\x19\x8e\xd5\xa1\xa1Z\xf4*\xb7\x22\xb5eU\xc2Z\x94\xba\xfe\x86\xb1\x04\x8b\x9a\x915\xc9\xb7\xe6,\x00\x1ceR\xfcY\xac4\x8e\xb9\x02JX\x1d\x9a\xc5Y\xe1W\xefs\xd1=(\xb5!\xc6bD{#~\xd0.\xb5$\xda\xb2]J\xa48\xeb\xb2\x1b\xe6(\x17Vt\x8eRqR\x05\xa1\xb7`q\xdd\xde`\x93o\x9d)\x06`\xb1Z\x0c&\x83X4\x8cb\x12\xdb\x1fP2{\x90\xce(\x13\x5c\x07\xeb\xea\xcf\xcf\xdb\xcb\x03?\xd9\x8c%08\x8e\xe1I$\xd3\x08\x84\x00\xc0S\x8e\x0a\x9b\x5c\x06\xa5\x0e\x9c\x95Og(\xa9\x10f\xa5\xac\x824Yy\x11\xfab,\xb1B\xb0\x05\x94[lcs\xd8\x14\xd4\xd4\xce\x07U\xf2\x1c\x0f\x8b\x11\x07w*\xe9\xf3|\x0fN<-\x13\x96\xb7\x18\xd1\xec\x95Whk\xc19yV\xca+5\xd2`\x06+\x165\x98\xde\x22\x7f\xc8\xde\x12+b\xa2\x0a*\x17\x90j+0\x8f\x17\x9ae\xecsH$\xc2\xcf\xdb\x98\x02!\xb5\xa8\xd2\xcb\xaa\xca\x04's\xd4uk\xccN,\xb2R\xc4\xc0\x22\xeb\xf2\xeb\x01=$Y\xb1X\x88'\xb1u\x95tY\x8a\x0dn:w6\x0f\x81\x90\x94\xe5W\x83\xb4\x9b7-:V\xb7\xd6\xe0\x97\xb9v\xd0\xcd\x16\x8c%\xbcx,M\xec\xcd\x04\x131\xce\x0aqV,\x16'E\x093>\xbb\x95\x1c\xfbj)\x1a\x0b\xa2{\xb00\xc4\xe8\x90\x5c\xdc\xdfTY\x93\xaf*\xa8\xeb\xff.\xc9\x06\x93\xf7\xb7\x14$Y\xb19(\xf3(vo%\xd5cuUD\xa1{Y\xe1\xeb\x1eT\x1e\xa0\x94@j\xd1rl\xf1\xe6o\xef\xdfVKy\xfe\x8a9\x8f\x95%\xd9\x91\x1c\xbe\x14\x80\xcd!\xec\xb0Yi\xb3[X\x1d\x8e=%\x1f\x90'\xdfs\xa5#\x08\xe8\x91\xd4\x16\xa3\x05W\x82\xb4\x1b\x80\xa7\xce\x93kZlv\x17u\xf1\xad|Y\x01\x81\xb8p\xe5\x00\x0c&\x83\xe0\x8c\x89\xda\x05\x04Pj\x95'\xa6c\x0fi\x98+N\xb0\x12\xaa`g\x13\xd1\xd0M\xe9$\xa3cu|Y7\x1e\x8d{_\xa1\xeb$G0rX\x915-\x9c\x99\xaf\x88\xe5\xf6\xba\xd7^TXD\xd2[!J\x0c\xabC\xc7\x1e4\x08\x8c%\x0c\x12\xed\x14K\x14&\xd3U\xcf\xa6\x0ef2\x5cm\xe3\xa2\xfd\xb3X\x9c\xbbv?O\xd7G\x8eT\xe4\xb0b#\x98\x85\xa6<\x10\x89.(/]\xe8\xad\xc0k\x87\xf2-\x87\xdb\x89\x13mt\x15=\x09\x9d:\x05S%\x12p\xcex\xb0\xde\xe6\xb4\xf3\x01\xb1\xaf\xbe\xfc/\xd4\xe5[\xad+\xdf_\xcez\x0ekD\xb1K\xe6 \xfa\xf46\x18\x07\xd7\x9a5j\x170\x0f\xae\x12\x9chC\x9f\x1f\x918LzT9\x04\xf6<\xc8\x82\xd0w\x08\x86\x95tN\x00\xab\xb5>\xf9H\xb7\xfb)cy\xe5.\xeao\xd4`\xe5g*<V\xaf\xb2Y\xdd2\xacp\xc6\xf4\x82\xd3\xe6\x80\xa3\xdc\x91\x9bp\xec\xa5u6\x04A]\xebv5\xea\xb7\x10%\x16%U\x96\x8a\x94BiE]i\x05\x8d\xc7\xbf\x0c\xfb*\xfd\xbf\xdao\xb1W\xcb.M\xb2\x99\x16.\x93]V\xa0<\xbcuZ\xb0\xa2\x12\xae\x12\xf9J\x22n\xa7\xcc\x9e\xb1\x8d\xc2j\xf3\xb1\x9a\x83\x22\x06v9\xeb\xc2\x99c3\x15\xb9\xacT\xba\xeb\xcb\x5c;4{>5hk\x10\xf5\x0e\xecf\x1c}\x02\xaf}\xa5\xd0{'\x94\xc0R\x9e\xb7\xc4\xb5&O\xc0\xe6\x91=\x9d\xdej:0\x1d\x9d\xe0\x95\x98\x8e5hq\xce\x93F`ux\xb1\x05\x16\xe3J]\x0b\xdeJ\xd5\xbb\xd5\xaa\xc7\x82\xc2\x96\xef\xbe\x0aU\xce%+\x8f?1~}ld\xd8[\xb7\x7f\xb3\x08J.\x22qDb\xb0\x18\x95\x84A\xd7\x19\xac\x11\xdb\x9f\xcd3\x1c\x85\xaag\xfc7\x89\xbcc\xdd\xb5)e^\xf1\x04\x9c\xf9\x9f\xb5P\xa6\x93\xb5\x0a\xc6\x92M]?w\xdd1;3z\xfb\xc6\xd9\xa1\xfe\xae\xdc\x5c\xea-\x0f\x1b\xd5\xb2\xc2\x1a\x05\x0f\xf8\x14\xc9?+\xdb\xf9\xff\xe2\xc2C\xe2\x18q\x05\x09\xc4\xf9p\xd6\x09:\xbd\x22\xac\x14\xbb\xc0\x1aU\x1ez\xf77\x80{\xbd\x17z\xba\x7f+X\xd1\xc9U]U\xdftDU\xefE\x8cX<E\x84\x95\x22\x06\xa5;\x15\x9c\xbf\x12\x8d\x04\x87\xfb?\x19\xf7\xf7e2)\x00\x06cqY\xc5v\xb3\xc5\xb1\x89\xbcg2L\x07\xef_\xee\xfc\xa5\xd81\xe2\x8erG\xe3SG\xccf\x15\x0bq\x00\x9c\xb5b\x99xZ\x9eU\x14\x18\xbd\xf3\xd1\xb97%\x0e\xa8\xaf\xdcR\xbf\xadn\xbf\xc7\xdb\xa2\xc1\x19\xb2\x85\xc4\xbd\xde\x0b\xdd\x97~%\xb6\x1d\xc7b\xb5\x1c\xfd\xc6\xd7\xaa\x9c'T\x9d\x0d'\xe4z\xad\x5c\x94\xba\xb3j\x0f\xd5\xb9^\xd1\xd9\xa0\xc4\x1e\xc0L:\x19\xf0\xdf\x09\xf8\xeft\x03\x8e\xd2\x1a\x8f\xb7\xe5\xf1=\xcfo6z2\xe9\xe4\xe5\xce_\x0e\xf4u\x8a50\x98\x0c_?\xf9\xa2!\xd3\xaa\xf6\xb8>\xd7n\xa5\xa7\xad\x01xt\x17!\xd2\xaa=\xb1X\xe8\xcc\x7f\xfe\x80j\xc7W\xa5\xbb\xbe\xa1\xe9\xf9\xba\x86\xb6u:\x93P\x12\xb33\xa3\x9d\xe7\xff]\xa2J\x91\xc5jy\xe1\x9f^\xd035v\xb3\xba\x13\xfb\xc4\xcf\x8e\xe2\xa1\xf1y\x91\xd7\xbb~\xd3{\xfd=\xf2\xc7\xe3a0\x15\xd75<\xdd\xf8\xe4\xcb\x1b(:\xb2'\xbb\xdb\x1c\xb6\x97\xbe\xf3R6Sl7P.g\xe5\xa1\x88\x81\xf7\xa8t\x161\xc1\xd9\xaa\xd11\x8c\x93f\xeed\xd2\xc9\x0b\xbf\xffi\xc0\xaf\xf0\x986^t\xd6\xff\xec\xc8\x81\xbe\xceK\x17OK4(\xab*{\xf9;/\xa7S\xac\x05/.\xe7K*Dy\x03J\x1f\x93nBv\x0eq\xe0\x06\xe6\xc8*\xde\x02\xb1X\xe8\xc2\xd9\x9fRW+\xcb\x81\xcd\xee:x\xf4\xfb\xaaNc\xa4A\xef\xf5\xf7\xaew\xfdF\xa2\x81\xb7\xc1\xdb\xfej{*\xc1\x99\xf0\xf5\xe5<|\x85 ;O\x9d\x8c\x15.\x85\x07\x9f\x90\x96F\x02\x92\x0b\xf3\x1f\xbc\xfb\x135\xc4\x00(s\xed8x\xf4{\x1a\x844$q\xf3\xca\x99\x9b\xddg\xc4\xae\xeaX]\xeb\xb3\xadM\x07\x9a\xd2)\x8eM~u9UQ!\x18=\xb6=M\xb2\x03\x82\xf8|\xfbx\x08\xa3W\xc8\x0b\xe9'\x17\xe6;\xcf\xff\xcc\xefS\x9b\xb4\xa8\xd9\xc9yB\xb8t\xf1\xb4\x84\xbbes\xd8\xda_mw\xd5\xb8\x16\xe6S&|M-% =F\x1d\x14\xac\x00\x08\xdd\xc7#\xba\xaa\x91\xd2_\x229*\xdd\xf5{\xf6\x1d\xd7V\xa7u\x9e\xff\x99D\xc1\xdf\xda\xc6\xda\xb6\x97\xda\x0c&C<Zd\xc6s|\xde\xa8*\x08\x9d\xaa&\x06\x1aV@g`x\xf8}=\x97.\x9c&*C-\x07\x9b\xdd\xd5\xda\xf6\xed\xda\x866\xf5]IH\x89\xc1d8\xf4\xfc\xa1\x9d\xcd;\x01$\xe6J\x8c\xd9Cj\xcd;H\xcd\xc92(Y\xe1R\xf0w\xd3\x86\x93\x93\x0b\xf3\xdd\x97\xde\x92\xd0\x15T\xa8mhkk\xff\xa1\x9a\xe4\x0d\x09J\xbc\x0d\xde\xc3/\x1d\xe6\xf3T2\xb1m\xba\x8c\x16\x9aSoA\xcdA\xaa\x0du\x94\xac\x00H\xc50\xdaMn\xf9\x97\xa1\xa1\xd08Jk\x9e;\xf6\x86\xb2\xc9\x8d\x98R\xb5X-G\xfe\xee\x88\xa7\xce\x03 \x9d\x04\x9b\xd9\x8f\xb4\xba0\x17\x0fF\x0f\xcfA\xa2\xddt9\xa0g\x05@r\x0e\x0f/+8\x01,\x93N\xf6^?\xdb{\xe3\xac\xfa\x8a/:\xd6\xf0L\xc7)Zm&6/i:\xd0\xb4\xf7\x99\xbd\x06\x93\x01@*n\xd3s\x07\xd4z\xc0<\x8a\x18T\x1f \xca\xe1\xce\xbbO\x09+\x00\x12a<\xbc\xac\xecl\xa3X,t\xa3\xeb7\x9a(\xb4#\x1d\xa7\xc8\xa7\x9c\x93\x81\x81\x0f\xdf\xfdI\xde\x07\xe1\xaaq\xb5\xbd\xd4\xc6\xe7\x1dr\x1c\x98T\x03\x92\x8aR\x87\x04Q\xbd\x1f\xc5J\x8a\x0b*e\x05\xc0|\x10\x81\x1b\x8a\x0f\x9d\x8aF\x82=\xddg\xd4sCH\xcc\xf2Qh\xcb\xbfX\xac\x96\x83\xcf\x1f\x5cN\xd6I\xc6-\x06\xaeU\x03\xf7w\x19\x95\xcd(Q\x98\xc2\xa1\x82\x15\xa8%\x06K\xdc\x8c\x0c]Ur\x9a\xdf\x12^\xfa\xd6\xbf\xc9Nh\xba?~k\xf9pw\x83\xc9\xd0t\xa0\xa9\xf9p3\x9fB\x9f\x8cg\xd9\xcc\x93LV\xd3\x10\x9c\x0aJ\xa0\x96\x15\x00\xf1\x10\xc6\xae\xa9<e2\x93N\x0e\xf5w\xf5\xdf\xf9\x83\xd8*\x934\xca\x5c;\x8e\x9f\xf8\xb9D\x83\xd9\x99\xd1\xdf\xfd\xfa\x87\xfc\xbfw\xed}\xbc\xe5\xe8^\xde\xcbJ\xa7\xb8lb\x9b>\xdb\xa8\x81\xef\xbb\x8c\x22\x06[\xf6\x12\xce\x16E\xfbP\xcb\x0a\x80D\x18\x81O\x15xek1;3\xdaw\xeb\x83\xa1\xfeOhEGZ\x8f\xf1\x91\xae\xda\xc6\xda\xd6g[\xf9\xfd7s\xe18\x17\xdfj/~JK>\x000z\xb8\xf7)0\xefy\xd0\x82\x15\x00\xa9\x18\xc6\xaeQ\x9c\x18\x22\x89L:\xe9\x1b\xba6x\xb7\x93<`\xb3\xf7\xe0k{\x0f\xbd&vud\xf8O\xf6\xaa\xb1\xb2\xaa\xb2Ppv>\x0c\xbbu\xb7\xdd\xba[\xac\xb1r\xb0FT\xef\xa7u\x82\x05\xa1\x11+\x00\xb8\x14\xc6{d\x92\xc7)\x91\x5c\x98\xf7\xfbzF\x1f\xf4\xf8\xef\xf7HOtZ\xdbN6\xb7\xbe\xaa\xe1\xd0\xd40;Q\xd5\xa2U\xed\x15\xedX\xe11\xf3\x05\xa6\x0aRq;\x1a\x09\x06\x03\x03\xc1\xf1\x81\xe9G#\x89\xc4\xfcr@\xdafwy\xeb\x0e\xb4\xb6\x9d\xdc\xc8\xd5L\xe7\x0e\x94?NU\xd3K\x1aZ\xb3\x02 \x1e\xc2\xf8\x8d\xff+YK\x8c\x1eU\xcd*m\xfbZ\x14\x80\x15\x14D\x9bmFh\xaa\xb5rQ\x18Vx\x84\xeeczp\xbd\x8ff_\x1f\x141(\xadC\xe9c\x1aj\xadU\xdd\x17\x90\x15\x00\xe9\x04\x1e\xddE\x94\xf2\xc4\xb1M\x8eb\x17*\x9e *~\xa3\x14\x05f\x85Gl\x1a\xc1\xcf\xb4\xf2\x9b7\x12\xac\x11\xae\xdd\x9a[\x91\xb5X\x17V\x00d9\x84\x8613\xfceUhE\x0c\x9c\xb5(\xad\x95\xaf\x15\xa1\xc9h\xeb\xc4\x0a\x0f.\x85\xb0\x1f\xa1\xa1/\x93\x87\xc6\xe8\xe1\xdc\x8e\x12\xefz\x96\x18^_Vxd9\xcc\xfa0;\xa2I\x90\xa6\x80`\x8d(\xd9\x06\xe7\x8e\xf5\x91\x8f\x5cl\x04+\xcb\x08\xfb\x11\x1dELi\x05\xfb\xc2\xc1X\x82\x92\xad(\xd9Z \x17K\x16\x1b\xca\x0a\x8fT\x0c\xd11DF7\xde\x1d\xd0[`\xab\x86\xbd\xa6\xa0\xfe\x15\x096\x01+\xcbH\x84\x11~\x88\xd8\xd4z\xd3\xa3\xb7\xc0R\x06\x9bG}\xacW+l&V\x96\x91N >\x85\xf8L\x01\x19\xe2\x990\x95\xc1R\xb6\x09O\xd5\xd8\x94\xac\xe4\x22\x9d@\x22\x8c\x85\x10\xd2q\xa4\xe2X\x08+\xf1\xad\x8b\x18\x98K\xa13\xc2P\x0cc\x09\x8c\xf6M\xc8D.\xfe\x17\xdd\x82\x10\xe7&\x87YM\x00\x00\x00\x00IEND\xaeB`\x82\x00\x00\x00\xf5<svg xmlns=\x22http://www.w3.org/2000/svg\x22 width=\x2224\x22 height=\x2224\x22 viewBox=\x220 0 24 24\x22><path d=\x22M0 0h24v24H0z\x22 fill=\x22none\x22/><path d=\x22M20 6h-8l-2-2H4c-1.1 0-1.99.9-1.99 2L2 18c0 1.1.9 2 2 2h16c1.1 0 2-.9 2-2V8c0-1.1-.9-2-2-2zm0 12H4V8h16v10z\x22/></svg>"
qt_resource_name = b"\x00\x0d\x02\x11\xfdg\x00u\x00g\x00a\x00o\x00_\x00i\x00c\x00o\x00n\x00.\x00p\x00n\x00g\x00\x0f\x04\x18\x9b\x87\x00f\x00o\x00l\x00d\x00e\x00r\x00_\x00o\x00p\x00e\x00n\x00.\x00s\x00v\x00g"
qt_resource_struct = b"\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00 \x00\x00\x00\x00\x00\x01\x00\x00\x1a\xf0"
def qInitResources():
    QtCore.qRegisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



import Manyo.Utsusemi as mu

tt=mu.UtsusemiDetectorEfficiencyCorrection()
tt.Initialize( 99999, "SIK-SUS304-3He-PSD" )
tt.MakeApproximateData( "SIK-SUS304-3He-PSD_10k.dat", 0.1, 10000, 1000, -50.0, 50.0, 1000 )
del tt

#dfile = "DetectorInfo_200302.xml"
#tt=mu.UtsusemiDetectorEfficiencyCorrection()
#tt.Initialize( str(dfile), "SIK-SUS304-3He-PSD" )
#tt.MakeApproximateData( "SIK-SUS304-3He-PSD_T0.dat", 0.1, 1000, 1000, -50.0, 50.0, 1000 )
#del tt




#!/usr/bin/python

from math import *
import numpy as np
import Manyo as m
import Functions

xmin = -5.0
xmax =  5.0
nDiv = 1000
p= [ 1.0, 0.0, 1.0 ]

gaussian = Functions.Gaussian()
delta = (xmax-xmin)/nDiv
for i in range(nDiv+1):
    x = xmin + delta*i
    print str(i).rjustify(4), str(x).rjustify(10), str(gauusian.eval(x, p)).rjustify(10)
